;    AnaCam.  Take anaglyph pictures on Android.
;    Copyright (C) 2012  Daniel Kraft <d@domob.eu>
;
;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

; Gimp script-fu to combine two images into an anaglyph.
; For simplicity, JPEG output quality is always set to 100.

; Save an image (with active layer) as JPEG with default options.
(define (saveJpeg img file)
  (let ((layer (car (gimp-image-get-active-layer img))))
    (file-jpeg-save RUN-NONINTERACTIVE img layer file file
                    1 ; Quality.
                    0 1 0 "" 0 1 0 0)))

; Given two input file names, produce an anaglyph.
(define (makeAnaglyph in1 in2 out)
  (let* ((img (car (gimp-file-load RUN-NONINTERACTIVE in1 in1)))
         (h (car (gimp-image-height img)))
         (w (car (gimp-image-width img)))
         (layerFile (car (gimp-file-load-layer RUN-NONINTERACTIVE img in2)))
         (layerRed (car (gimp-layer-new img w h RGB-IMAGE "red"
                                        100 SCREEN-MODE)))
         (layerCyan (car (gimp-layer-new img w h RGB-IMAGE "cyan"
                                         100 SCREEN-MODE)))
         (colourRed '(255 0 0))
         (colourCyan '(0 255 255)))

    (gimp-image-add-layer img layerCyan -1)
    (gimp-context-set-foreground colourCyan)
    (gimp-drawable-fill layerCyan FOREGROUND-FILL)

    (gimp-image-add-layer img layerFile -1)

    (gimp-image-add-layer img layerRed -1)
    (gimp-context-set-foreground colourRed)
    (gimp-drawable-fill layerRed FOREGROUND-FILL)

    (let ((newUpper (car (gimp-image-merge-down img layerRed CLIP-TO-IMAGE)))
          (newLower (car (gimp-image-merge-down img layerCyan CLIP-TO-IMAGE))))
      (gimp-layer-set-mode newUpper MULTIPLY-MODE)
      (gimp-image-merge-down img newUpper CLIP-TO-IMAGE)
      (saveJpeg img out)
      (gimp-image-delete img))))
